/**************************************************************
	LCD Serial communication monitor (inexpensive)
	ACM0802C  8 colimns 2 lines
	SC1602BS 16 columns 2 lines (Default)
	SC242A	 24 columns 2 lines
	 for PIC16F1823
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	Tools:
	MPLAB v8.73
	Hitech PIC C pro9.82
---------------------------------------------------------------
	This program is new header for Hitech PICC.
	When you use an old compiler, you will have to correct some descriptions. 
---------------------------------------------------------------
Date	:Time	Version	Build	Note
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20110621:2300	1.00	---		begin the program
20110622:1700	1.10	---		FIX CCP1 output
20110623:0800	1.20	---		Corresponding MPLAB ver8.70 and HTC ver9.81
20110625:0100	1.30	---		Corresponding MPLAB ver8.70 and HTC ver9.82
20110625:0800	1.31	---		wrong lcd message was fixed.
20110720:0800	1.32	---		Ring buffer adjust.
20111010:2300	1.33	---		Corresponding MIDI speed(31250bps).
---------------------------------------------------------------
	Pin assignment
	1:VDD:P:2.7V TO 5V
	2:RA5:I:RIGHT SW
	3:RA4:I:LEFT SW
	4:RA3/MCLR:I:MODE SW/PICKIT2-1(W/10kPU)
	5:RC5:O:P PULSE OUT(IF YOU NEED)
	6:RC4:0:TX(NOT USED)
	7:RC3:O:LCD D7

	14:VSS:Power GND
	13:RA0:O:LCD E
	12:RA1:I:RX
	11:RA2:O:LCD RS
	10:RC0:O:LCD D4
   	 9:RC1:0:LCD D5
	 8:RC2:O:LCD D6
---------------------------------------------------------------
	Display cursor 
	ASCII mode : cursor is blinking (always can switch with command Ctrl-Q/W)
	Binary mode: cursor is stacking (under ber)

	Behavior when code is received
	ascii code	12 		(ctrl +l)	LCD clear
	ascii code	0x0b	(ctrl +k)	Toggle line
	ascii code	0x0d	(Enter)		Return command
	ascii code	0x11	(ctrl +q)	Cursor off
	ascii code	0x17	(ctrl +w)	Cursor on
	ascii code	0x20 to 0xff		Display the corresponding ascii code
---------------------------------------------------------------
Be careful!
	This use LCD character module(SC242A(24 x 2 lines)) is different to power connection of SC1602 series.
	Please careful that the direction of module power polar. 
	
**************************************************************/
//#define _LEGACY_HEADERS
#include <htc.h>
#include "sci.h"
#include "lcd.h"
#include "eeprom.h"	//***1.33
#define MHz 000000
#define _XTAL_FREQ 8MHz

//Version infomation
#define	VersionCode	"ver1.33"
#ifdef	_LCD0802
#define	VersionInfo	"ACM0802C"
#endif
#ifdef	_LCD1602
#define	VersionInfo	"SC1602BS "
#endif
#ifdef	_LCD2402
#define	VersionInfo	"SC242A(16F1823) "
#endif


#define	SW_MODE		RA3 //mode set
#define	SW_LEFT		RA4 //left key
#define	SW_RIGHT	RA5 //right key
#define DisplayMode	0x7f //eeprom display address
#define COM_Speed	0x7e //eeprom display address

//speed
#define Speed1200	249
#define Speed2400	250
#define Speed4800	251
#define Speed9600	252
#define Speed19200	253
#define Speed31250	254	// ***1.33
#define Speed38400	255

#define MIN_Speed	Speed1200	//***1.33
#define MAX_Speed	Speed38400	//***1.33


#define ASCIImode	0xff
#define HEXmode		0xfe




#define MAX_BUFF	73	//Ring buffer ***1.32

//__CONFIG
__CONFIG(
	FOSC_INTOSC & WDTE_OFF & PWRTE_ON & MCLRE_OFF 
	& CP_OFF & CPD_OFF & BOREN_ON & CLKOUTEN_OFF 
	& IESO_OFF & FCMEN_OFF
	);

__CONFIG(
	WRT_OFF & PLLEN_OFF & STVREN_ON & LVP_OFF
//	& BORV_27 & DEBUG_OFF	//should be comment line,if use HitechPICC ver9.81.**1.20
//	& BORV_25	//more and equal HItechPICC ver 9.81  **1.20
	& BORV_LO	//more and equal HItechPICC ver 9.83  **1.33
	);


//Global settings
//Fifo Data
unsigned char Buffer_FIFO[MAX_BUFF] @ 0x2000;	//***1.32
unsigned char FIFO_write_pointer=0;
unsigned char FIFO_read_pointer=0;
unsigned char FIFO_datasize=0;

void interrupt handler( void )	//interrupt handle
{
	unsigned char temp;
	unsigned char data;
	//Handle RS232 Recieve interrupt
	if( RCIF == 1 ){
		//recive to data
		data = sci_GetByte();
		//set to FIFO
		RCIF = 0; //clear RS232 Recieve interrupt bit
		if(FIFO_datasize<MAX_BUFF){
			Buffer_FIFO[FIFO_write_pointer]=data;
			FIFO_write_pointer=(FIFO_write_pointer+1) & (MAX_BUFF-1);//ring buff 
			FIFO_datasize++;
		}
	}
	if( TMR0IF==1 ){	//timer interval interrupt
		if (!SW_LEFT){
			while(!SW_LEFT);// button release check
			asm("RESET");//reset
		}
	
		if (!SW_RIGHT)	{
			temp = E2promRead(DisplayMode);
			if (temp != ASCIImode ) E2promWrite(DisplayMode,ASCIImode);
				else E2promWrite(DisplayMode,HEXmode);
			while(!SW_RIGHT);// button release check
			asm("RESET");//reset
		}

		if (sci_CheckOERR()) { //sci error
			CREN=1;
		}

	//timer interval
	TMR0IF =0 ;
	}
}

unsigned char Buffer_readdata(void){
	unsigned char wdat;
	//read FIFO buffer
	while(FIFO_datasize==0);	// pause if buffer is empty
	wdat=Buffer_FIFO[FIFO_read_pointer];
	FIFO_read_pointer=(FIFO_read_pointer+1)&(MAX_BUFF-1);//ring buff
	FIFO_datasize--;
	return wdat;//get wdat
//	return FIFO_datasize;//debug
}	

void Buffer_check(void){
	unsigned char data;
	if (FIFO_datasize==MAX_BUFF-1) {
		lcd_putch(0xFF);
		lcd_puts("Full");
		lcd_putch(0xFF);
		RCIE = 0;	//terminal interrupt disable
		FIFO_datasize 		=0;	//FIFO clear
		FIFO_write_pointer 	=0;
		FIFO_read_pointer 	=0;
		CREN =0;	//Receive was aborted
		while (RCIF ==1){
			data = sci_GetByte();		//Dummy read
		}
		__delay_ms(50);
		__delay_ms(50);
		__delay_ms(50);
		__delay_ms(50);
		CREN = 1;	//Receive is start
		RCIE = 1;	//terminal interrupt enable				
	}	
}

void delay_10ms(unsigned int temp){	//10ms multiple x times
	while(temp--){
		__delay_ms(10);
	}	
}


//------------------main
main()
{
// Register
	unsigned int speed = 38400; //default communication speed set
	unsigned char temp,data,buffspeed;
//	Port initalize
//		  	  76543210
	PORTA = 0b11100000;	//Port Initalize
	PORTC = 0b00000000;	//Port Initalize
	TRISA = 0b11111010; //
	TRISC = 0b11010000; //ccp1 output
	ANSELA	= 0x00;		//All port are use for digital port
	ANSELC	= 0x00;		//All port are use for digital port
	CM1CON0 = 0x07;		//PORTA is all digital I/O
	RXDTSEL	= 1;		//RA0 pin assignment as RXD
	OSCCON	= 0b01110010;	// PLL off,8MHz,IntOSC

//Init
	OPTION_REG = 0b00000111; // prescale 1/256 RBPU= EN
	GIE  = 0;//Int off

//timer0 setting (interrupt event timer 3.328ms)
	TMR0IE = 0;//timer interrupt is off
	TMR0 = 0;//interrupt timer counter set

//PWM initialize setting (83.8KHz on RB3 @8MHz)

	CCP1CON	=	0b00001100;	//Single output,LSBset,PWM mode: P1A, P1C active-high; P1B, P1D active-high
	PR2 = 0x17; // resolution is 6.5bit mode
	CCPR1L = 0x0c; // duty is 50%
	T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1

/*
//PWM test (31.25KHz on RB3 @8MHz) test only
	CCP1CON = 0b00001100;// use PWM mode
	PR2 = 0xff; // resolution is 8bit mode
	CCPR1L = 0x20; // duty is 50%
	T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1
*/

//LCD init
//	DelayMs(200);	//stabilize time
	delay_10ms(20);
	lcd_init();		//lcd initalize
	lcd_clear();	//lcd clear
//setting mode
	if (!SW_MODE ){
		lcd_write(LCD_CursorOff);
		lcd_goto(LCD_LineNumber1);
		lcd_puts(VersionInfo);	//version kind of LCD
		lcd_puts(VersionCode);	//version code
		while(!SW_MODE);
		lcd_clear();
		lcd_goto(LCD_LineNumber2);
#ifndef	_LCD0802
		lcd_puts("Set  DOWN  UP");
#else
		lcd_puts("Set -  +");	//**1.31
#endif	


		buffspeed = E2promRead(COM_Speed);
		while(1){
			__delay_ms(50);		//DelayMs(50);
			lcd_goto(LCD_LineNumber1);
			switch (buffspeed){
				case Speed1200:
					lcd_puts(" 120");
					break;
				case Speed2400:
					lcd_puts(" 240");
					break;
				case Speed4800:
					lcd_puts(" 480");
					break;
				case Speed9600:
					lcd_puts(" 960");
					break;
				case Speed19200:
					lcd_puts("1920");
					break;
				case Speed31250:
					lcd_puts("3125");	//***1.33
					break;
				case Speed38400:
					lcd_puts("3840");
					break;
				default: // data is broken
					buffspeed =Speed38400;
			}

			lcd_puts("0bps");
		
			//key check
			if (!SW_LEFT ) {
				if ( buffspeed > MIN_Speed) buffspeed--;
				while(!SW_LEFT);

			}
			if (!SW_RIGHT  ){
				if ( buffspeed < MAX_Speed) buffspeed++;
				while(!SW_RIGHT);

			}
			if (!SW_MODE  ){
				if (buffspeed != E2promRead(COM_Speed)){
					E2promWrite(COM_Speed,buffspeed);
					lcd_goto(LCD_LineNumber2);
					lcd_puts("OK ");
					lcd_write(LCD_CursorBlink);
					//DelayMs(255);
					delay_10ms(50);	
				}
				while(!SW_MODE);
				break;
			}
		}
	}

	buffspeed = E2promRead(COM_Speed);
//sci initialize
	switch (buffspeed){
		case Speed1200:
			speed = 1200;
			break;
		case Speed2400:
			speed = 2400;
			break;
		case Speed4800:
			speed = 4800;
			break;
		case Speed9600:
			speed = 9600;
			break;
		case Speed19200:
			speed = 19200;
			break;
		case Speed31250:	//***1.33
			speed = 31250;
			break;
		case Speed38400: //This speed is not accept HEX mode.
			speed = 38400;
			break;
		default:
			speed = 38400;
	}

	sci_Init(speed ,SCI_EIGHT);// Baud set and Bit set

//interrupt enable
//	INTEDG = 0; //RB0 Key interrupt fall edge
//	INTE = 1;//RB0 is interrupt mode
	TMR0IE = 1;//timer interrupt on
	RCIE = 1;//recive interrupt on
	PEIE = 1;//Peripheral interrupt Enable 
	GIE  = 1;//Global interrupt Enable 

//sci check
	if (sci_CheckOERR()) { //sci error
		while (!TRMT);
		CREN=1;
	}

//Main routiene
	lcd_init();
	lcd_clear();	//lcd clear

/*
//test
	temp = 0x20;
	while(1){
		if ( temp == 0xff ) temp = 0x20;
		lcd_putch(temp++);
		delay_10ms(20);
	}
	//test end	
*/

	temp = E2promRead(DisplayMode);

	switch (temp){
		case HEXmode :
			lcd_write(LCD_CursorOn);	//Init cursor
			//ASCII to HEX convert(Use less or equal 19200bps)
//			lcd_puts("HEX:");
//			lcd_goto(LCD_LineNumber2);
			while(1)	{
				data= Buffer_readdata();//Buffer read 
			    //Send high nibble
			    if( (data>>4) > 9 )
			        lcd_putch( 'A' - 10 + (data>>4) );
			    	else lcd_putch( '0' + (data>>4) );
			    //Send low nibble
			    if( (data&0x0F) > 9 )
			        lcd_putch( 'A' - 10 + (data&0x0F) );
			    	else lcd_putch( '0' + (data&0x0F) );

				Buffer_check();	//Buffer check
			}
			break;	

		default:
			lcd_write(LCD_CursorBlink);	//Init cursor
//			lcd_puts("ASCII:");
//			lcd_goto(LCD_LineNumber2);
			while(1){
				data= Buffer_readdata();//Buffer read 
				switch ( data ){
					case 12	:	//clear
						lcd_clear();
						break;
					case 0x0b	:	//Verticcal tab
						lcd_newline(1);
						break;
					case 0x0d	:	//return command
						lcd_newline(0);
						break;
					case 0x11	:	//cursor off(ctrl +q)
						lcd_write(LCD_CursorOff);//cursor off
						break;
					case 0x17	:	//cursor on(ctrl +w)
						lcd_write(LCD_CursorBlink);//cursor on
						break;
					default		:
						if 	(( data >= 0x20 ) && ( data <= 0xff  )) lcd_putch(data);
						break;
				}
			}
		break;

	}

}
